@@ -721,15 +721,14 @@ MOUNT-BELOW specifies a pathname to prefix to mount points when opening
 FILESYSTEM volumes.  OPENED-VOLUMES specifies a symbol to which a list of all
 volumes that were opened will be bound, which can be used to do things like
 populate /etc/fstab and /etc/crypttab.  Do not modify this list."
-  (once-only (mount-below)
-    (let ((opened-volumes (or opened-volumes (gensym))))
-      `(let ((,opened-volumes (open-volumes-and-contents
-                               ,volumes
-                               ,@(and mount-below-supplied-p
-                                      `(:mount-below ,mount-below)))))
-         (unwind-protect (progn ,@forms)
-           (mrun "sync")
-           (mapc #'close-volume ,opened-volumes))))))
+  (let ((opened-volumes (or opened-volumes (gensym))))
+    `(let ((,opened-volumes (open-volumes-and-contents
+                             ,volumes
+                             ,@(and mount-below-supplied-p
+                                    `(:mount-below ,mount-below)))))
+       (unwind-protect (progn ,@forms)
+         (mrun "sync")
+         (mapc #'close-volume ,opened-volumes)))))
 
 (defmacro with-these-open-volumes
     ((volumes &key (mount-below nil mount-below-supplied-p)) &body propapps)
@@ -751,18 +750,16 @@ must not be modified."
     (volumes propapp &key (mount-below nil mount-below-supplied-p))
   (:retprop
    :type (propapptype propapp)
-   :hostattrs (lambda (&rest ignore)
-                (declare (ignore ignore))
+   :hostattrs (lambda-ignoring-args
                 (require-volumes-data volumes)
                 (propappattrs propapp))
    :apply
-   (lambda (&rest ignore)
-     (declare (ignore ignore))
+   (lambda-ignoring-args
      (with-connattrs (:opened-volumes
                       (apply #'open-volumes-and-contents
                              `(,volumes ,@(and mount-below-supplied-p
                                                `(:mount-below ,mount-below)))))
-       (unwind-protect-in-parent (propappapply propapp)
+       (unwind-protect (propappapply propapp)
          (mrun "sync")
          (mapc #'close-volume (get-connattr :opened-volumes)))))
    :args (cdr propapp)))
@@ -825,6 +822,7 @@ the LVM physical volumes corresponding to those volume groups."
 (defproplist caches-cleaned :posix ()
   "Clean all caches we know how to clean in preparation for image creation."
   (:desc "Caches cleaned")
+  (file:data-cache-purged)
   (os:typecase
     (debianlike (apt:cache-cleaned))))
 
@@ -889,6 +887,10 @@ the LVM physical volumes corresponding to those volume groups."
      ;; Finally, create the volumes.
      (create-volumes-and-contents volumes))))
 
+(defun image-chroot (image-pathname)
+  (ensure-directory-pathname
+   (strcat (unix-namestring image-pathname) ".chroot")))
+
 (defpropspec raw-image-built-for :lisp
     (options host image-pathname &key rebuild)
   "Build a raw disk image for HOST at IMAGE-PATHNAME.
@@ -914,8 +916,7 @@ Unless REBUILD, the image will not be repartitioned even if the specification
 of the host's volumes changes, although the contents of the image's
 filesystems will be incrementally updated when other properties change."
   (:desc #?"Built image for ${(get-hostname host)} @ ${image-pathname}")
-  (let ((chroot (ensure-directory-pathname
-                 (strcat (unix-namestring image-pathname) ".chroot")))
+  (let ((chroot (image-chroot image-pathname))
         (volumes
           (loop
             with found
@@ -924,10 +925,9 @@ filesystems will be incrementally updated when other properties change."
             if (and physical-disk-p (not found)
                     (slot-boundp volume 'volume-contents))
               do (setq found t)
-              and collect (let ((copy (copy-volume-and-contents volume)))
-                            (change-class copy 'raw-disk-image)
-                            (setf (image-file copy) image-pathname)
-                            copy)
+              and collect (aprog1 (copy-volume-and-contents volume)
+                            (change-class it 'raw-disk-image)
+                            (setf (image-file it) image-pathname))
             else unless physical-disk-p
                    collect volume
             finally
@@ -942,6 +942,111 @@ filesystems will be incrementally updated when other properties change."
        (consfigurator.property.installer:chroot-installed-to-volumes
         ,host ,chroot ,volumes))))
 
+(defprop %squashfsed :posix (chroot image &optional (compression "xz"))
+  (:apply
+   (file:does-not-exist image)
+   (with-remote-temporary-file (excludes)
+     (writefile excludes
+                (format nil "~@{~&~A~}" "/boot" "/proc" "/dev" "/sys" "/run"))
+     (run :inform "nice" "mksquashfs" chroot image
+          "-no-progress" "-comp" compression "-ef" excludes))))
+
+;; Based on live-wrapper, and some help from this guide:
+;;     <https://willhaley.com/blog/custom-debian-live-environment/>
+(defpropspec debian-live-iso-built :lisp (options image-pathname properties)
+  "Build a Debian Live hybrid ISO at IMAGE-PATHNAME for a host with properties
+PROPERTIES, which should specify, at a minimum, the operating system for the
+live system.  OPTIONS is a plist of keyword parameters:
+
+  - :CHROOT-OPTIONS -- passed on to CHROOT:OS-BOOTSTRAPPED-FOR, which see.
+
+Currently only BIOS boot is implemented."
+  (:desc #?"Debian Live ISO built @ ${image-pathname}")
+  (destructuring-bind
+      (&key chroot-options
+       &aux (chroot (image-chroot image-pathname))
+         (iso-root (ensure-directory-pathname
+                    (strcat (unix-namestring image-pathname) ".cd")))
+         (isolinux (merge-pathnames "isolinux/" iso-root))
+         (squashfs (merge-pathnames "live/filesystem.squashfs" iso-root))
+         (host (make-host
+                :hostattrs '(:hostname ("debian"))
+                :propspec
+                (append-propspecs
+                 properties
+                 (make-propspec
+                  :propspec
+                  '(eseqprops
+                    (apt:installed "initramfs-tools" "linux-image-amd64"
+                     "live-boot" "task-laptop" "libnss-myhostname"
+                     "syslinux-common" "isolinux")
+                    (caches-cleaned))))))
+         (host-arch (os:linux-architecture (get-hostattrs-car :os host))))
+      options
+    (unless (member host-arch '(:amd64))
+      (inapplicable-property
+       "Architecture ~A of live host not supported." host-arch))
+    `(eseqprops
+      (apt:installed "squashfs-tools" "xorriso")
+      (file:directory-exists ,isolinux)
+      (file:containing-directory-exists ,squashfs)
+      (on-change (chroot:os-bootstrapped-for ,chroot-options ,chroot ,host)
+
+        (%squashfsed ,chroot ,squashfs)
+
+        ;; Copy the chroot's versions of bootloader binaries.
+        (file:is-copy-of ,(merge-pathnames "isolinux.bin" isolinux)
+                         ,(chroot-pathname "/usr/lib/ISOLINUX/isolinux.bin"
+                                           chroot))
+        ,@(loop for basename in '("ldlinux" "libcom32" "vesamenu" "libutil"
+                                  "libutil" "libmenu" "libgpl" "hdt")
+                for file = (strcat basename ".c32")
+                collect
+                `(file:is-copy-of
+                  ,(merge-pathnames file isolinux)
+                  ,(chroot-pathname
+                    (merge-pathnames file "/usr/lib/syslinux/modules/bios/")
+                    chroot)))
+
+        ;; Copy the targets of the symlinks in the root of the chroot.
+        (file:is-copy-of ,(merge-pathnames "live/vmlinuz" iso-root)
+                         ,(merge-pathnames "vmlinuz" chroot))
+        (file:is-copy-of ,(merge-pathnames "live/initrd.img" iso-root)
+                         ,(merge-pathnames "initrd.img" chroot))
+
+        (file:exists-with-content ,(merge-pathnames "isolinux.cfg" isolinux)
+          ("UI vesamenu.c32"
+           ""
+           "MENU TITLE Live Boot Menu"
+           "DEFAULT linux"
+           "TIMEOUT 600"
+           "MENU RESOLUTION 640 480"
+           ""
+           "LABEL linux"
+           "  MENU LABEL Debian Live [BIOS/ISOLINUX]"
+           "  MENU DEFAULT"
+           "  KERNEL /live/vmlinuz"
+           "  APPEND initrd=/live/initrd.img boot=live"
+           ""
+           "LABEL linux"
+           "  MENU LABEL Debian Live [BIOS/ISOLINUX] (nomodeset)"
+           "  MENU DEFAULT"
+           "  KERNEL /live/vmlinuz"
+           "  APPEND initrd=/live/initrd.img boot=live nomodeset"))
+
+        (cmd:single
+         :inform
+         "xorriso" "-as" "mkisofs" "-iso-level" "3" "-o" ,image-pathname
+         "-full-iso9660-filenames" "-volid" "DEBIAN_LIVE"
+         "-isohybrid-mbr" ,(chroot-pathname "/usr/lib/ISOLINUX/isohdpfx.bin"
+                                            chroot)
+
+         "-eltorito-boot" "isolinux/isolinux.bin"
+         "-no-emul-boot" "-boot-load-size" "4" "-boot-info-table"
+         "--eltorito-catalog" "isolinux/isolinux.cat"
+
+         ,iso-root)))))
+
 (defprop host-volumes-created :lisp ()
   "Recursively create the volumes as specified by DISK:HAS-VOLUMES.
 **THIS PROPERTY UNCONDITIONALLY FORMATS DISKS, POTENTIALLY DESTROYING DATA,
